import pytest
import responses

from brian_dashboard_manager.grafana.provision import provision_folder, provision

TEST_INTERFACES = [
    {
        "router": "srx2.ch.office.geant.net",
        "name": "ge-0/0/8",
        "bundle": [],
        "bundle-parents": [],
        "description": "PHY CUSTOMER GEANT CORPORATE SRF000001 | GEANT Corporate to MX1.LON - Via Vodafone",  # noqa: E501
        "circuits": [
            {
                "id": 679232,
                "name": "GEANT CORPORATE TO MX1.LON - VIA VODAFONE",
                "type": "GEANT IP",
                "status": "non-monitored"
            }
        ],
        "snmp-index": 523,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "GEANT",
            "interface_type": "PHYSICAL"
        },
        "dashboards_info": [{
            "name": "GEANT",
            "interface_type": "PHYSICAL"
        }],
        "ipv4": [],
        "ipv6": []
    },
    {
        "router": "srx2.ch.office.geant.net",
        "name": "ge-0/0/8.10",
        "bundle": [],
        "bundle-parents": [],
        "description": "SRV_GLOBAL CUSTOMER GEANT #GEANT_CORPORATE-ViaVodafone | GEANT Corporate to mx1.lon - Via Vodafone ",  # noqa: E501
        "circuits": [
            {
                "id": 679360,
                "name": "GEANT_CORPORATE-VIAVODAFONE",
                "type": "GEANT IP",
                "status": "non-monitored"
            }
        ],
        "snmp-index": 543,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "GEANT",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "GEANT",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [
            "62.40.127.131/31"
        ],
        "ipv6": [
            "2001:798:99:1::ca/126"
        ]
    },
    {
        "router": "srx2.ch.office.geant.net",
        "name": "ge-0/0/8.11",
        "bundle": [],
        "bundle-parents": [],
        "description": "SRV_GLOBAL CUSTOMER GEANT #GEANT_CORPORATE_ViaVodafone-VRF | GEANT Corporate to mx1.lon - Via Vodafone - for VRF",  # noqa: E501
        "circuits": [
            {
                "id": 712144,
                "name": "GEANT_CORPORATE_VIAVODAFONE-VRF",
                "type": "GEANT IP",
                "status": "non-monitored"
            }
        ],
        "snmp-index": 545,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "GEANT",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "GEANT",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [
            "10.101.0.6/30"
        ],
        "ipv6": []
    },
    {
        "router": "srx2.ch.office.geant.net",
        "name": "ge-0/0/8.12",
        "bundle": [],
        "bundle-parents": [],
        "description": "SRV_GLOBAL CUSTOMER GEANT #GEANT_CORPORATE_ViaVodafone-VRF-TEST | GEANT Corporate to mx1.lon - Via Vodafone - DASHBOARD BGP TEST VLAN",  # noqa: E501
        "circuits": [
            {
                "id": 678920,
                "name": "GEANT_CORPORATE_VIAVODAFONE-VRF-TEST (DO NOT OPEN A TICKET)",  # noqa: E501
                "type": "GEANT IP",
                "status": "non-monitored"
            }
        ],
        "snmp-index": 522,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "GEANT",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "GEANT",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [
            "62.40.127.141/31"
        ],
        "ipv6": []
    },
    {
        "router": "srx2.ch.office.geant.net",
        "name": "ge-0/0/8.996",
        "bundle": [],
        "bundle-parents": [],
        "description": "SRV_GLOBAL CUSTOMER GEANT #GEANT_OPERATIONS_LabConnectivity | GEANT MX1.LON Infinera VRF to Operations Lab",  # noqa: E501
        "circuits": [
            {
                "id": 678999,
                "name": "GEANT_OPERATIONS_LABCONNECTIVITY",
                "type": "GEANT IP",
                "status": "non-monitored"
            }
        ],
        "snmp-index": 554,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "GEANT",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "GEANT",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [
            "10.1.1.1/30"
        ],
        "ipv6": []
    },
    {
        "router": "mx1.ams.nl.geant.net",
        "name": "xe-0/0/0",
        "bundle": [
            "ae16"
        ],
        "bundle-parents": [],
        "description": "PHY CUSTOMER KIAE P_AE16 SRF9915671 |",
        "circuits": [],
        "snmp-index": 588,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "KIAE",
            "interface_type": "PHYSICAL"
        },
        "dashboards_info": [{
            "name": "KIAE",
            "interface_type": "PHYSICAL"
        }],
        "ipv4": [],
        "ipv6": []
    },
    {
        "router": "mx1.ams.nl.geant.net",
        "name": "xe-0/0/1",
        "bundle": [
            "ae14"
        ],
        "bundle-parents": [],
        "description": "PHY CUSTOMER SWITCH P_AE14 SRF18008 |",
        "circuits": [],
        "snmp-index": 589,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "SWITCH",
            "interface_type": "PHYSICAL"
        },
        "dashboards_info": [{
            "name": "SWITCH",
            "interface_type": "PHYSICAL"
        }],
        "ipv4": [],
        "ipv6": []
    },
    {
        "router": "mx1.ams.nl.geant.net",
        "name": "ge-0/2/1.0",
        "bundle": [],
        "bundle-parents": [],
        "description": "SRV_L2CIRCUIT INFRASTRUCTURE GEANT GEANT #AMS-ZAG OPENFLOW |",  # noqa: E501
        "circuits": [],
        "snmp-index": 687,
        "dashboards": [
            "L2_CIRCUIT"
        ],
        "dashboard_info": {
            "name": "GEANT",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "GEANT",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [],
        "ipv6": []
    },
    {
        "router": "mx1.ams.nl.geant.net",
        "name": "ge-0/2/2.0",
        "bundle": [],
        "bundle-parents": [],
        "description": "SRV_L2CIRCUIT INFRASTRUCTURE GEANT GEANT #AMS-VIE OPENFLOW |",  # noqa: E501
        "circuits": [],
        "snmp-index": 711,
        "dashboards": [
            "L2_CIRCUIT"
        ],
        "dashboard_info": {
            "name": "GEANT",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "GEANT",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [],
        "ipv6": []
    },
    {
        "router": "mx1.ams.nl.geant.net",
        "name": "ge-0/2/4.0",
        "bundle": [],
        "bundle-parents": [],
        "description": "SRV_L2CIRCUIT INFRASTRUCTURE GEANT GEANT #AMS-FRA OPENFLOW |",  # noqa: E501
        "circuits": [],
        "snmp-index": 718,
        "dashboards": [
            "L2_CIRCUIT"
        ],
        "dashboard_info": {
            "name": "GEANT",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "GEANT",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [],
        "ipv6": []
    }
]

NREN_INTERFACES = [
    {
        "router": "mx1.dub2.ie.geant.net",
        "name": "xe-0/0/0",
        "bundle": [
                "ae10"
        ],
        "bundle-parents": [],
        "description": "PHY CUSTOMER HEANET P_AE10 SRF9948758 | HEANET-AP2-LL3",  # noqa: E501
        "circuits": [],
        "snmp-index": 554,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "HEANET",
            "interface_type": "PHYSICAL"
        },
        "dashboards_info": [{
            "name": "HEANET",
            "interface_type": "PHYSICAL"
        }],
        "ipv4": [],
        "ipv6": []
    },
    {
        "router": "mx1.dub2.ie.geant.net",
        "name": "xe-1/0/1",
        "bundle": [
                "ae10"
        ],
        "bundle-parents": [],
        "description": "PHY CUSTOMER HEANET P_AE10 SRF0000001 | HEANET-AP2-LL2",  # noqa: E501
        "circuits": [],
        "snmp-index": 527,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "HEANET",
            "interface_type": "PHYSICAL"
        },
        "dashboards_info": [{
            "name": "HEANET",
            "interface_type": "PHYSICAL"
        }],
        "ipv4": [],
        "ipv6": []
    },
    {
        "router": "mx1.dub2.ie.geant.net",
        "name": "xe-1/1/0",
        "bundle": [
                "ae10"
        ],
        "bundle-parents": [],
        "description": "PHY CUSTOMER HEANET P_AE10 SRF9925903 | HEANET-AP2-LL1",  # noqa: E501
        "circuits": [],
        "snmp-index": 528,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "HEANET",
            "interface_type": "PHYSICAL"
        },
        "dashboards_info": [{
            "name": "HEANET",
            "interface_type": "PHYSICAL"
        }],
        "ipv4": [],
        "ipv6": []
    },
    {
        "router": "mx1.dub2.ie.geant.net",
        "name": "ae10",
        "bundle": [],
        "bundle-parents": [
                "xe-0/0/0",
                "xe-1/0/1",
                "xe-1/1/0"
        ],
        "description": "LAG CUSTOMER HEANET SRF9925909 | HEANET-AP2-LAG",
        "circuits": [],
        "snmp-index": 596,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "HEANET",
            "interface_type": "AGGREGATE"
        },
        "dashboards_info": [{
            "name": "HEANET",
            "interface_type": "AGGREGATE"
        }],
        "ipv4": [],
        "ipv6": []
    },
    {
        "router": "mx1.dub2.ie.geant.net",
        "name": "ae10.12",
        "bundle": [],
        "bundle-parents": [
                "xe-0/0/0",
                "xe-1/0/1",
                "xe-1/1/0"
        ],
        "description": "SRV_GLOBAL CUSTOMER HEANET #HEANET-AP2 | ASN1213 | ",
        "circuits": [
            {
                "id": 662976,
                "name": "HEANET-AP2",
                "type": "GEANT IP",
                "status": "operational"
            }
        ],
        "snmp-index": 713,
        "dashboards": [
            "NREN",
            "RE_CUST"
        ],
        "dashboard_info": {
            "name": "HEANET",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "HEANET",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [
            "62.40.125.13/30"
        ],
        "ipv6": [
            "2001:0798:0018:10aa::15/126"
        ]
    },
    {
        "router": "mx1.dub2.ie.geant.net",
        "name": "ae10.30",
        "bundle": [],
        "bundle-parents": [
                "xe-0/0/0",
                "xe-1/0/1",
                "xe-1/1/0"
        ],
        "description": "SRV_MDVPN CUSTOMER HEANET AP2 #HEANET-BGP-LU-CoC-1 |",  # noqa: E501
        "circuits": [
            {
                "id": 663160,
                "name": "HEANET-BGP-LU-COC-1",
                "type": "MD-VPN (NATIVE)",
                "status": "operational"
            }
        ],
        "snmp-index": 712,
        "dashboards": [
            "MDVPN_CUSTOMERS",
            "NREN"
        ],
        "dashboard_info": {
            "name": "HEANET",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "HEANET",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [
            "62.40.102.36/31"
        ],
        "ipv6": []
    },
    {
        "router": "mx1.dub2.ie.geant.net",
        "name": "ae10.333",
        "bundle": [],
        "bundle-parents": [
                "xe-0/0/0",
                "xe-1/0/1",
                "xe-1/1/0"
        ],
        "description": "SRV_IAS CUSTOMER HEANET #HEANET-AP2-IAS IASPS | ASN1213 ",  # noqa: E501
        "circuits": [
            {
                "id": 663214,
                "name": "HEANET-AP2-IAS",
                "type": "GEANT PEERING",
                "status": "operational"
            }
        ],
        "snmp-index": 658,
        "dashboards": [
            "IAS_CUSTOMER",
            "NREN"
        ],
        "dashboard_info": {
            "name": "HEANET",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "HEANET",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [
            "83.97.88.77/30"
        ],
        "ipv6": [
            "2001:798:1::61/126"
        ]
    },
    {
        "router": "mx1.dub2.ie.geant.net",
        "name": "ae10.1214",
        "bundle": [],
        "bundle-parents": [
                "xe-0/0/0",
                "xe-1/0/1",
                "xe-1/1/0"
        ],
        "description": "SRV_L2CIRCUIT CUSTOMER HEANET GEANT #ams-dub2-HEANET-RARE-21061 |",  # noqa: E501
        "circuits": [
            {
                "id": 713335,
                "name": "AMS-DUB2-HEANET-RARE-21061",
                "type": "GEANT PLUS",
                "status": "non-monitored"
            }
        ],
        "snmp-index": 790,
        "dashboards": [
            "L2_CIRCUIT"
        ],
        "dashboard_info": {
            "name": "HEANET",
            "interface_type": "LOGICAL"
        },
        "dashboards_info": [{
            "name": "GEANT",
            "interface_type": "LOGICAL"
        },
            {
            "name": "HEANET",
            "interface_type": "LOGICAL"
        }],
        "ipv4": [],
        "ipv6": []
    }
]

EUMETSAT_MULTICAST = [
    {
        'router': 'mx1.ams.nl.geant.net',
        'oid': '1.3.6.1.2.1.83.1.1.2.1.16.232.223.222.1.193.17.9.3.255.255.255.255',  # noqa: E501
        'community': '0pBiFbD',
        'subscription': '232.223.222.1',
        'endpoint': '193.17.9.3'
    },
    {
        'router': 'mx1.ams.nl.geant.net',
        'oid': '1.3.6.1.2.1.83.1.1.2.1.16.232.223.222.2.193.17.9.3.255.255.255.255',  # noqa: E501
        'community': '0pBiFbD',
        'subscription': '232.223.222.2',
        'endpoint': '193.17.9.3'
    },
    {
        'router': 'mx1.lon.uk.geant.net',
        'oid': '1.3.6.1.2.1.83.1.1.2.1.16.232.223.222.1.193.17.9.3.255.255.255.255',  # noqa: E501
        'community': '0pBiFbD',
        'subscription': '232.223.222.1',
        'endpoint': '193.17.9.3'
    },
    {
        'router': 'mx1.lon.uk.geant.net',
        'oid': '1.3.6.1.2.1.83.1.1.2.1.16.232.223.222.2.193.17.9.3.255.255.255.255',  # noqa: E501
        'community': '0pBiFbD',
        'subscription': '232.223.222.2',
        'endpoint': '193.17.9.3'
    },
    {
        'router': 'mx1.fra.de.geant.net',
        'oid': '1.3.6.1.2.1.83.1.1.2.1.16.232.223.222.1.193.17.9.3.255.255.255.255',  # noqa: E501
        'community': '0pBiFbD',
        'subscription': '232.223.222.1',
        'endpoint': '193.17.9.3'
    },
    {
        'router': 'mx1.fra.de.geant.net',
        'oid': '1.3.6.1.2.1.83.1.1.2.1.16.232.223.222.2.193.17.9.3.255.255.255.255',  # noqa: E501
        'community': '0pBiFbD',
        'subscription': '232.223.222.2',
        'endpoint': '193.17.9.3'
    }
]


def generate_folder(data):
    return {
        "id": 555,
        "uid": data['uid'],
        "title": data['title'],
        "url": f"/dashboards/f/{data['uid']}/{data['title'].lower()}",
        "hasAcl": False,
        "canSave": True,
        "canEdit": True,
        "canAdmin": True,
        "createdBy": "Anonymous",
        "created": "2021-02-23T15:33:46Z",
        "updatedBy": "Anonymous",
        "updated": "2021-02-23T15:33:46Z",
        "version": 1
    }


@pytest.fixture
def reporting_provider(get_test_data, data_config):
    responses.add(
        method=responses.GET,
        url=f"{data_config['reporting_provider']}/scid/current",
        json=get_test_data("services.json"),
    )


@pytest.fixture
def populate_inventory(data_config):
    """function-fixture for provisioning inventory provider. Call it with a dictionary
    {url_path: contents}. ie {"/poller/interfaces": [...]}
    """

    def _populate(contents_dict):
        for path, contents in contents_dict.items():
            responses.add(
                method=responses.GET,
                url=f"{data_config['inventory_provider']}{path}",
                json=contents,
            )

    return _populate


@responses.activate
@pytest.mark.parametrize(
    "folder_name, excluded_nrens, expected_nrens",
    [
        ("NREN Access", [], ["GEANT", "KIAE", "SWITCH"]),
        ("NREN Access", ["GEANT", "KIAE"], ["SWITCH"]),
        (
            "NREN Access BETA",
            [],
            ["ASNET-AM", "LITNET", "CESNET", "GEANT", "KIAE", "SWITCH"],
        ),
        (
            "NREN Access BETA",
            ["ASNET-AM", "GEANT"],
            ["LITNET", "CESNET", "KIAE", "SWITCH"],
        ),
        ("testfolder", ["GEANT"], ["KIAE", "SWITCH"]),
    ],
)
def test_provision_nren_folder(
    folder_name,
    excluded_nrens,
    expected_nrens,
    data_config,
    mock_grafana,
    reporting_provider,
    populate_inventory,
):
    dashboards = {
        "NREN": {
            "tag": ["customers"],
            "folder_name": "NREN Access",
            "interfaces": [
                iface for iface in TEST_INTERFACES if "NREN" in iface["dashboards"]
            ],
        },
        "RE_CUST": {
            "tag": "RE_CUST",
            "folder_name": "RE Customer",
            "interfaces": [
                iface for iface in TEST_INTERFACES if "RE_CUST" in iface["dashboards"]
            ],
        },
    }
    populate_inventory(
        {
            "/poller/interfaces": NREN_INTERFACES,
            "/data/interfaces": NREN_INTERFACES,
            "/poller/eumetsat-multicast": EUMETSAT_MULTICAST,
        }
    )

    result = provision_folder(
        mock_grafana.request,
        folder_name,
        dashboards["NREN"],
        data_config,
        "testdatasource",
        excluded_nrens,
    )
    assert len(result) == len(expected_nrens)
    for i, nren in enumerate(expected_nrens):
        assert result[i]["title"] == nren
        if "NREN" in folder_name:
            # Every NREN dashboard must have at least 4 panels
            # (3 default panels and 1 per ifc)
            assert len(result[i]["panels"]) > 3


@responses.activate
def test_provision(
    data_config, mocker, mock_grafana, reporting_provider, populate_inventory
):
    mock_grafana.create_datasource(
        {
            "name": "brian-influx-datasource",
            "type": "influxdb",
            "access": "proxy",
            "url": "http://test-brian-datasource.geant.org:8086",
            "database": "test-db",
            "basicAuth": False,
            "isDefault": True,
            "readOnly": False,
        }
    )
    populate_inventory(
        {
            "/poller/interfaces": NREN_INTERFACES,
            "/data/interfaces": NREN_INTERFACES,
            "/poller/eumetsat-multicast": EUMETSAT_MULTICAST,
        }
    )
    for org in data_config["organizations"][1:]:
        mock_grafana.create_organization(org)

    mock_grafana.create_dashboard({"title": "testdashboard", "version": 1})

    _mocked_gws = mocker.patch(
        'brian_dashboard_manager.grafana.provision.get_gws_direct')
    _mocked_gws.return_value = []

    _mocked_gws_indirect = mocker.patch(
        'brian_dashboard_manager.grafana.provision.get_gws_indirect')
    _mocked_gws_indirect.return_value = []

    provision(data_config, raise_exceptions=True)


@responses.activate
def test_provision_re_peer_dashboard(
    mocker, data_config, mock_grafana, reporting_provider, populate_inventory
):
    interfaces = [
        {
            "router": "mx1.dub2.ie.geant.net",
            "name": "xe-0/0/0.1",
            "description": "PHY SVC P_AE10 SRF9948758 | HEANET-AP2-LL3",  # noqa: E501
            "dashboards": ["RE_PEER"],
            "dashboard_info": {"name": "ESNET", "interface_type": "LOGICAL"},
            "dashboards_info": [{"name": "ESNET", "interface_type": "LOGICAL"}],
            "ipv4": ["1.1.1.1"],
            "ipv6": ["::2"],
        },
    ]
    populate_inventory(
        {
            "/poller/interfaces": interfaces,
            "/data/interfaces": interfaces,
            "/poller/eumetsat-multicast": EUMETSAT_MULTICAST,
        }
    )
    _mocked_gws = mocker.patch(
        "brian_dashboard_manager.grafana.provision.get_gws_direct"
    )
    _mocked_gws.return_value = []

    _mocked_gws_indirect = mocker.patch(
        "brian_dashboard_manager.grafana.provision.get_gws_indirect"
    )
    _mocked_gws_indirect.return_value = []
    data_config["organizations"] = [
        {"name": "Testorg1", "excluded_nrens": ["GEANT"], "excluded_dashboards": []},
    ]
    provision(data_config, raise_exceptions=True)
    folder_uid = "RE_Peer"
    assert len(mock_grafana.dashboards_by_folder_uid[folder_uid]) == 1
    panels = mock_grafana.dashboards_by_folder_uid[folder_uid][0]["panels"]
    expected_types = ["text", "graph", "graph", "row", "graph", "graph", "row"]
    assert [p["type"] for p in panels] == expected_types
    assert "INFO" in panels[0]["title"]
    assert "ingress" in panels[1]["title"]
    assert "egress" in panels[2]["title"]
    assert "Services" in panels[3]["title"]
    assert "traffic" in panels[4]["title"]
    assert "IPv6" in panels[5]["title"]
    assert "Interfaces" in panels[6]["title"]
